require "trick/function"

RSpec.describe Trick::Function do
  def args
    [:a, "b", 1, nil, [], {}]
  end

  describe "::to_proc" do
    it "returns a Proc" do
      expect(ABC.to_proc).to be_a Proc
    end

    it "returns callable with arguments" do
      expect(ABC.to_proc.call(*args)).to eq ABC.call(*args)
    end

    it "applies as a block" do
      expect(
        ["A", "B", "C"].map(&Letter)
      ).to eq ["Letter A", "Letter B", "Letter C"]
    end

    it "applies curried proc as a block" do
      expect(points.map(&NormalizedDistance[5])).to eq [10, 200]
    end
  end

  describe "::curry" do
    it "returns a curried Proc" do
      expect(Clean["mop"]).to be_a Proc
    end

    it "returns a result of curried Proc's applying" do
      expect(Clean["mop"].("hall")).to eq "Clean hall with mop"
    end
  end

  describe "::call" do
    it { expect(ABC).to respond_to(:call) }

    it "accepts any arguments" do
      expect { ABC.call(*args) }.not_to raise_error
    end

    it { expect(ABC.call).to eq nil }
  end
end
