require "trick/function"
require "trick/value"

module ABC
  extend Trick::Function
end

module Letter
  extend Trick::Function

  def call(x)
    "Letter #{x}"
  end
end

module Clean
  extend Trick::Function

  def call(tool, room)
    "Clean #{room} with #{tool}"
  end
end


A = Struct.new :x, :y do
  extend Trick::Value
end

class RoundedNum
  extend Trick::Value

  attr_reader :value

  def initialize(number)
    @value = number.round
  end
end

class Sum
  extend Trick::Value

  attr_reader :sum

  def initialize(a, b)
    @sum = a + b
  end
end


# README
Point = Struct.new :coords do
  extend Trick::Value

  def radius
    Math.sqrt(coords[0]**2 + coords[1]**2)
  end
end

def points
  @points ||= [[3, 4], [8, 6]].map(&Point)
end

module NormalizedDistance
  extend Trick::Function

  # main function method
  def call(divisor, point)
    point.radius / divisor * (inc_order?(point) ? 10 : 100)
  end

  # use private methods in your function
  private def inc_order?(point)
    point.coords[1] > point.coords[0]
  end
end
