require "trick/value"

RSpec.describe Trick::Value do
  describe "::to_proc" do
    it "acts as a constructor of a class" do
      expect(
        [5.75, 0.99].map(&RoundedNum).map(&:value)
      ).to eq [6, 1]
    end

    it "constructs instances", :readme do
      expect(points).to eq [Point.new([3, 4]), Point.new([8, 6])]
    end

    it "constructs instances with ability to pass messages to them", :readme do
      expect(points.map(&:radius)).to eq [5, 10]
    end
  end

  describe "::curry" do
    context "Struct" do
      it "returns a struct class", :readme do
        expect(A[1]).to be_a Proc
      end

      it "returns a result of instantiating a struct" do
        expect(A[1, 2]).to eq A[1, 2]
      end
    end

    context "class", :readme do
      it "returns a curried Proc" do
        expect(Sum[41]).to be_a Proc
      end

      it "returns a result of curried Proc's applying" do
        expect(Sum[1, 2].sum).to eq 3
      end
    end
  end

  describe "::call" do
    it "returns new instance of a class" do
      expect(A.(10, 17)).to be_a A
    end

    it "creates an instance with passed values" do
      a = A.(1, 2)

      expect(a.x).to eq 1
      expect(a.y).to eq 2
    end

    it "creates an instance with passed value" do
      expect(RoundedNum.(3.333).value).to eq 3
    end
  end
end
